#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.141592653589793
#define SWAP(a,b) {float temp=(a);(a)=(b);(b)=temp;}
#define MAXST 80
float f(float);

						 /*  MODULE 5.8       */
void chebpc(float c[], float d[], int n)
                                                                      /*
5.8 POLYNOMINAL COEFFICIENTS FROM A CHEBYSHEV FIT

VERSION  1 04/Sept/92 Krakow updated by Tomasz Twardowski

Given a coefficient array c[0..n-1], this routine generates a
coefficient array d[0..n-1] such that
(k=0,n-1) d(k+1)y^k = (k=0,n-1) c(k+1)Tk(y)
The method is Clenshaw's recurrence, but now applied
algebraically rather than arithmetically.                             */
{
int      j, k;
float    sv, *dd;
                                                                      /*
------------------------------------------ CHEBPC - BEGIN ----------- */

    dd = vector(0, n - 1);
    for (j = 0; j < n; j++)
       d[j] = dd[j] = 0.0;
    d[0] = c[n - 1];
    for (j = n - 2; j >= 1; j--)
    {
       for (k = n - j; k >= 1; k--)
       {
          sv = d[k];
          d[k] = 2.0 * d[k - 1] - dd[k];
          dd[k] = sv;
       }
       sv = d[0];
       d[0] = -dd[0] + c[j];
       dd[0] = sv;
    }
    for (j = n - 1; j >= 1; j--)
       d[j] = d[j - 1] - dd[j];
    d[0] = -dd[0] + 0.5 * c[0];
    free_vector(dd, 0, n - 1);
                                                                      /*
------------------------------------------ CHEBPC - END ------------- */
}

                                                 /*  MODULE 5.6       */
void chebft(float a, float b, float c[], int n)
                                                                      /*
5.6 FIT A CHEBYSHEV POLYNOMINAL TO A FUNCTION

VERSION  1 04/Sept/92 Krakow updated by Tomasz Twardowski

Given a function func, lower and upper limits of the intervals [a,
b], and a maximum degree n, this routine computes the n coeffi- n-1
cients c[0..n-1] such that func(x)[ ckTk(y)] - c0/2 . k=1 This
routine is to be used with moderately large n, the array of c's
subsequently to be truncated at the smaller value m such that cm and
subsequent elements are neghgible.                                    */
{
int      j, k;
float    fac, bpa, bma, *func, *vector(int , int );
void     free_vector(float *, int , int );
                                                                      /*
------------------------------------------ CHEBFT - BEGIN ----------- */

    func = vector(0, n - 1);
    bma = 0.5 * (b - a);
    bpa = 0.5 * (b + a);
                                         /*  we evaluate the function
      at the n points required by (5.6.7, p.167 "Numerical Recipes")  */
    for (k = 0; k < n; k++)
    {
       float    y = cos(PI*(k+0.5)/n);

       func[k] = f(y * bma + bpa);
    }
    fac = 2.0 / n;
    for (j = 0; j < n; j++)
    {
       double   sum = 0.0;
                                                         /*  we will 
accumulate the sum in double precision a nicety which you can ignore  */
       for (k = 0; k < n; k++)
          sum += func[k] * cos(PI * j * (k + 0.5) / n);
       c[j] = fac * sum;
    }
    free_vector(f, 0, n - 1);
                                                                      /*
------------------------------------------ CHEBFT - END ------------- */
} 

                                                 /*  MODULE 2.1       */
void gaussj(float a[][MAXST+2], int n, float b[][2], int m)
                                                                      /*
2.1 LINEAR EQUATION SOLUTION BY GAUSS-JORDAN ELIMINATION.

VERSION  1 04/Sept/92 Krakow  updated by Tomasz Twardowski

The input matrix a[n][n] has n by n elements. b[n][m] is an matrix 
of size n by m containing the m right-hand side vectors. On output, 
a is replaced by its matrix inverse, and b is replaced by the 
corresponding set of solution vectors.                                */
{
int      *indxc, *indxr, *ipiv;
int      i, icol, irow, j, k, l, ll;//, *ivector();
float    big, dum, pivinv;
//void     nrerror(), free_ivector();
                             /*  the integer arrays ipiv[n],indxr[n] 
and indxc[n] are used for bookkeeping on the pivoting.                */
                                                                      /*
------------------------------------------ GAUSSJ - BEGIN ----------- */

    indxc = ivector(1, n);
    indxr = ivector(1, n);
    ipiv = ivector(1, n);
    for (j = 1; j <= n; j++)
       ipiv[j] = 0;
            /*  this is the main loop over the columns to be reduced  */
    for (i = 1; i <= n; i++)
    {
       big = 0.0;
       /*  this is the outer loop of the search for a pivot element.  */
       for (j = 1; j <= n; j++)
          if (ipiv[j] != 1)
             for (k = 1; k <= n; k++)
             {
                if (ipiv[k] == 0)
                {
                   if (fabs(a[j][k]) >= big)
                   {
                      big = fabs(a[j][k]);
                      irow = j;
                      icol = k;
                   }
                }
                else
                   if (ipiv[k] > 1)
                      nrerror("GAUSSJ: Singular Matrix-1");
             }
       ++(ipiv[icol]);
/*  we now have the pivot element, so we interchange rows, if needed, 
to put the pivot element on the diagonal. the columns are not 
physically interchanged, only relabeled: indxc[i], the column of the 
ith pivot element, is the ith column that is reduced, while indxr[i] 
is the row in which that pivot element was originally located. 
if indxr[i]!=indxc[i] there is an implied column interchange. 
with this form of bookkeeping, the solution b's will end up in the 
correct order, and the inverse matrix will be scrambled by columns.   */
       if (irow != icol)
       {
          for (l = 1; l <= n; l++)
             SWAP(a[irow][l], a[icol][l]);
          for (l = 1; l <= m; l++)
             SWAP(b[irow][l], b[icol][l]);
       }
                                             /*  we are now ready to 
divide the pivot row by the pivot element, located at irow and icol.  */
       indxr[i] = irow;
       indxc[i] = icol;
       if (a[icol][icol] == 0.0)
          nrerror("GAUSSJ: Singular Matrix-2");
       pivinv = 1.0 / a[icol][icol];
       a[icol][icol] = 1.0;
       for (l = 1; l <= n; l++)
          a[icol][l] *= pivinv;
       for (l = 1; l <= m; l++)
          b[icol][l] *= pivinv;
   /*  next we reduce the rows, except for the pivot one, of course.  */
       for (ll = 1; ll <= n; ll++)
          if (ll != icol)
          {
             dum = a[ll][icol];
             a[ll][icol] = 0.0;
             for (l = 1; l <= n; l++)
                a[ll][l] -= a[icol][l] * dum;
             for (l = 1; l <= m; l++)
                b[ll][l] -= b[icol][l] * dum;
          }
    }
                   /*  this is the end of the main loop over columns 
of the reduction. it only remains to unscramble the solution in 
view of the column interchanges. we do this by interchanging pairs 
of columns in the reverse order that the permutation was built up.    */
    for (l = n; l >= 1; l--)
    {
       if (indxr[l] != indxc[l])
          for (k = 1; k <= n; k++)
             SWAP(a[k][indxr[l]], a[k][indxc[l]]);
    }
    free_ivector(ipiv, 1, n);
    free_ivector(indxr, 1, n);
    free_ivector(indxc, 1, n);
                                                                      /*
------------------------------------------ GAUSSJ - END ------------- */
} 

float f(float x) {
	return exp(2.0*x)/2.0;
}

void indeksuj(float*x, float*y, int n) {
	int i;
	
	for (i=0; i<n; i++) {
		x[i] = -1.0 + 2.0*(float)(i)/(float)(n-1);
		y[i] = f(x[i]);
	}
}

float potega(float x,int n) {
  if (n<1) return 1.0;
  return x*potega(x,n-1);
}

int main() {
	float *xw, *yw;
	int ST;
	int i,j,k,n;
  	float *a, *x, *wynik;
  	float Wolne[MAXST+1], Skladnik[MAXST*2], Suma[MAXST*2], Tablica[MAXST+1][MAXST+2];
  	float TablicaA[MAXST+2][MAXST+2], TablicaB[MAXST+2][2];

	printf("Podaj ilo wzw oraz stopie wielomianu aproksymujcego:\n");
	scanf("%d %d", &n, &ST);
	
	xw = (float*)malloc(sizeof(float)*n);
	yw = (float*)malloc(sizeof(float)*n);
	
	indeksuj(xw, yw, n);

  	for(j=0;j<(ST+1);j++) {
    		Wolne[j]=0;
    		for(k=0;k<(ST+2);k++)
    			Tablica[j][k]=0;
  	}
  	for(j=0;j<(2*ST);j++) {
    		Skladnik[j]=0;
    		Suma[j]=0;
  	}
  	for(j=0;j<n;j++) {
    		Skladnik[0]=xw[j];
    		for(i=1;i<(2*ST);i++)
 			Skladnik[i]=xw[j]*Skladnik[i-1];
    		for(i=0;i<(2*ST);i++)
 			Suma[i]+=Skladnik[i];
    		Wolne[0]+=yw[j];
    		for(i=1;i<(ST+1);i++)
 			Wolne[i]+=yw[j]*Skladnik[i-1];
  	}
  	Tablica[0][0]=n;
  	for(i=0;i<(ST+1);i++) {
    		Tablica[i][ST+1]=Wolne[i];
    		for(j=0;j<(ST+1);j++) {
			k=i+j;
      			if(k!=0)
				Tablica[i][j]=Suma[k-1];
    		}
  	}
  	for(i=0;i<(ST+1);i++)
    		for(j=0;j<(ST+1);j++)
      			TablicaA[i+1][j+1]=Tablica[i][j];
  	for(i=0;i<(ST+1);i++)
    		TablicaB[i+1][1]=Tablica[i][ST+1];
  	gaussj(TablicaA,ST+1,TablicaB,1);
	
  	printf("Wielomian aproksymacji sredniokwadratowej:\n");
  
  	for (i=ST; i>=0; i--) {
    		if ((i!=ST) && (TablicaB[i+1][1]>=0))
			printf("+");
    		printf("%.3f",TablicaB[i+1][1]);
    		if (i>0)
			printf("*x");
    		if (i>1)
			printf("**%i",i);
  	}
  	printf("\n");
     
  	a = (float*) vector(0, ST);
  	x = (float*) vector(0, ST);
	wynik = (float*) vector(0, ST);
	chebft( -1, 1, a, ST+1);
	chebpc(a, wynik, ST+1);
  	printf("Wielomian aproksymacji jednostajnej:\n");
   
  	for(i=ST;i>=0;i--) {
      		if (i!=ST && wynik[i]>=0)
			printf("+");
      		printf("%f",wynik[i]);
      		if (i>0)
			printf("*x");
      		if (i>1)
			printf("**%i",i);
  	}
      
  	printf("\n");
  	free_vector(a, 0, ST);
  	free_vector(x, 0, ST);
  	free_vector(wynik, 0, ST);
  
  	return EXIT_SUCCESS;
}
