#define MAX 100
#define FLOAT

#include <stdlib.h>
#include <stdio.h>

#include "basis.h"
#include "glsp.h"

void splinecoeffns(int n,float x[],float f[],float a[][MAX],int *st){

/*  The procedure calculates the coefficents of the natural cubic
		spline interpolant for a function given by its values at nodes.
    Data:
      n - number of interpolant nodes minus 1 (nodes numbered from 0 
          to n),
      x - an array containing the values of nodes,
      f - an array containing the values of function.
    Result:
      a - an array (matrix) of spline coefficents (the element a[k,i]
          contains the coefficent before x^k, where k=0,1,2,3, for the
          interval <x[i], x[i+1]>; i=0,1,...,n-1).
    Parameters:
      st - a variable which within the procedure is assigned the value of:
             1 if n<1,
						 2 if there exist x[i] and x[j] (i<>j; i,j=0,1,...,n) such
							 that x[i]=x[j].
						 0 otherwise.
		Note: If st<>0, then the elements of matrix are not calculated.  */


	int i,k;
	float u,v,y,z,xi;
	float *d;
	float *b;
	float *c;

	d=(float *)calloc(n+1,sizeof(float));
	b=(float *)calloc(n-1,sizeof(float)); 	 /* elements used from 1 to n-2   */
	c=(float *)calloc(n,sizeof(float));		 /* elements used from 2 to n-1   */

	if (n<1){*st=1;}
	else{
		*st=0;
		i=-1;
		do{
			i++;
      for(k=i+1;k<=n;k++){if(x[i]==x[k]) *st=2;}
    }while(i!=(n-1) && *st!=2);
	}
  if(*st==0){
		d[0]=0;
    d[n]=0;
		if(n>1){
      z=x[2];
      y=z-x[1];
      z=z-x[0];
      u=f[1];
      b[1]=y/z;
      d[1]=6*((f[2]-u)/y-(u-f[0])/(x[1]-x[0]))/z;
			z=x[n-2];
      y=x[n-1]-z;
      z=x[n]-z;
      u=f[n-1];
      c[n-1]=y/z;
      d[n-1]=6*((f[n]-u)/(x[n]-x[n-1])-(u-f[n-2])/y)/z;
      for(i=2;i<=n-2;i++){
        z=x[i];
        y=x[i+1]-z;
				z=z-x[i-1];
				u=f[i];
        b[i]=y/(y+z);
				c[i]=1-b[i];
        d[i]=6*((f[i+1]-u)/y-(u-f[i-1])/z)/(y+z);
      }
      u=2;i=0;
      y=d[1]/u;
      d[1]=y;
      if(n>2){
				do{
          i++;
          z=b[i]/u;
          b[i]=z;
          u=2-z*c[i+1];
          y=(d[i+1]-y*c[i+1])/u;
          d[i+1]=y;
        }while(i!=(n-2));
      }
      u=d[n-1];
			for(i=n-2;i>=1;i--){
        u=d[i]-u*b[i];
				d[i]=u;
      }
      for(i=0;i<=n;i++){
        u=f[i];
        xi=x[i];
        z=x[i+1]-xi;
        y=d[i];
				v=(f[i+1]-u)/z-(2*y+d[i+1])*z/6;
        z=(d[i+1]-y)/(6*z);
        y=y/2;
        a[0][i]=((-z*xi+y)*xi-v)*xi+u;
        u=3*z*xi;
        a[1][i]=(u-2*y)*xi+v;
        a[2][i]=y-u;
        a[3][i]=z;
      }    
		}
	}
	free(b);free(d);free(c);
}

float f(float x) {
	return x/(2.0 + x*x*x);
}

void indeksuj(float*x, float*y, int n) {
	int i;
	for (i=0; i<n; i++) {
		x[i] = 0.0 + 4.5*((float)i/(n-1));
		y[i] = f(x[i]);
	}
	x[n] = 5;
}

int main() {
	int st=0;
	float x[MAX];
  	float y[MAX];
  	float mac[4][MAX];
	float w[MAX];
	float a[MAX], b[MAX], c[MAX], d[MAX];
  	int i, j;
	int n, ret = EXIT_SUCCESS;
	
	printf("Podaj ilosc wezlow: ");
	scanf("%d", &n);
	
	indeksuj(x,y,n);

	for (i=0; i<MAX; i++)
		w[i]=99999999999999999999;

  	splinecoeffns(n-1,x,y,mac,&st);
	if(st==0) {
		printf("Funkcja interpolujaca:\n");
		printf("plot [0:4.5] x/(2+x**3), ");
		for(j=0;j<=n-2;j++) {
			printf("(abs(x-%f)<%f? ", j*(4.5/(n-1))+2.25/(n-1), 2.25/(n-1));
			for (i=3;i>=0;i--) {
				if ((mac[i][j]>=0) && (i<3))
					printf("+");
				printf("%.4f",mac[i][j]);
				if (i>1)
					printf("*x**%d",i);
				else if (i==1)
					printf("*x");
			}
			printf(":1/0)");
			if (j<n-2)
				printf(", ");
		}
		printf("\n\n");
	} else {
		fprintf(stderr, "Interpoacja: nie obliczono!\n");
		ret = EXIT_FAILURE;
	}
	
	if (glspnp(n-1, x, y, w, 1, 0.5, -0.0207847, a, b, c, d)) {
		fprintf(stderr, "Aproksymacja: nie obliczono! (err %d)\n", glspnp(n-1, x, y, w, 1, 0.5, -0.0207847, a, b, c, d));
		ret = EXIT_FAILURE;
	} else {
		printf("Funkcja aproksymujaca:\nplot [0:4.5] x/(2+x**3), ");
		for(i=0;i<=n-2;i++) {
			printf("(abs(x-%f)<=%f? ", i*(4.5/(n-1))+2.25/(n-1), 2.25/(n-1));
			printf("%f*(x-%f)**3", d[i], x[i]);
			if (c[i]>=0)
				printf("+");
			printf("%f*(x-%f)**2", c[i], x[i]);
			if (b[i]>=0)
				printf("+");
			printf("%f*(x-%f)", b[i], x[i]);
			if (a[i]>=0)
				printf("+");
			printf("%f", a[i]);
			printf(":1/0)");
			if (i<n-2)
				printf(", ");
		}
		printf("\n");
	}
	
	return ret;
}
